/* STL Exporter version 1.1 for Lightwave 9.x - 11.5
   By Chilton Webb (chiltonwebb@gmail.com). Please email me if you have any problems, requests, or feedback.

   Based on works by Sean Hyde-Moyer (c) 2003
   This program is released to the public under the terms of the
   GNU GENERAL PUBLIC LICENSE
   See http://www.gnu.org/copyleft/gpl.txt
   For the full text of the license agreement

 Changes in version 2.0
 * Updated LScript commands for compatibility with 9.x through 11.5
 * Switched to ASCII only.
 * Got rid of Mac/PC issues
 * Fixes invert issues by default to maximize compatibility
 * Added more robust error checking instead of deleting random parts of user's model
 * Added base unit scale, assuming output requires mm support.
 * Added rotation for x axis, so finished products are oriented in the same direction as they are in LW
 * Corrected undo calculation so *hopefully* there is no difference to your model after using this script.
 * Fixed some other stuff I forgot to pay attention to
 * Updated notes in lscript so the user knows what they're getting into.
 * Added reference notes in the stl file so you can remember where you put the original file by reading the STL.
 * Added better error checking for filename conflicts and other issues.

*/

@script modeler


main
{

	baseScale = 1;
	pScale[1] = "Meters";
	pScale[2] = "Millimeters";
	multiplier = 1.0;


//______Prep File to Write_______________________________________________________________


    directory = recall ("directory","Drive");

	reqbegin("Export STL");
    	reqsize(500,200);

    	YPOS = 10; 

		f1 = ctlfilename("Export To:",directory,100,0);
        		ctlposition(f1,10,YPOS,450,20);
			YPOS = YPOS + 25;

		t3 = ctlchoice("Modeler's Default Unit:",baseScale,pScale);
        		ctlposition(t3,10,YPOS,230,20);
			YPOS = YPOS + 55;


		w1 = ctltext("","Notes: Save before using, as geometry changes *might* occur. Undo to get rid of them, if so.");
        		ctlposition(w1,10,YPOS,450,20);
			YPOS = YPOS + 21;

		w2 = ctltext("","Also, keep objects on one layer only. Assumes printer base unit is millimeters.");
        		ctlposition(w2,10,YPOS,450,20);
			YPOS = YPOS + 30;
			
			
		b2 = ctltext("","STL Export v2.0 -- Problems? Email chiltonwebb@gmail.com");
        	ctlposition(b2,120,YPOS,450,20);
        		

	if(reqpost())
	{
		filename = getvalue(f1);
		invertIt = 1;

		whichScale = getvalue(t3);
		if (whichScale==1) {
			multiplier = 1000.0;
		} 


	} else {
		return;
	}
	
	reqend();

	if (filename == nil) {
		return;
	}

	base = split(filename);

	stlname = base[3];



	//_____________________Make the extension STL


	newFilename = string(base[1],base[2],"/",base[3],".stl");

	if(fileexists(newFilename)){

		reqbegin("STL Export");
    		reqsize(300,80);

    		YPOS = 10; 

		w1 = ctltext("Warning! File Exists.","Press OK to overwrite, or CANCEL to quit.");
        ctlposition(w1,30,YPOS,280,20);
		if(reqpost())
		{

		}
		else
			return;
		reqend();

	}


	stlFile = File(newFilename,"w");


	if(stlFile == nil){
		error("Cannot Create File \"",filename,"\"");
		return;
	}

    store("directory",filename);



//______Prep File For STL Compatibility

	testbadpolys = polycount(); // Delete 1 & 2 point polygons
	if((testbadpolys[2] + testbadpolys[3]) > 0)
	{
		selmode(USER);
		selpolygon(SET,NVLT,3);
		error("One or Two Point Polys detected. Please get rid of these and try again.");
		return;
	}

	selmode(USER); 
	selpolygon(CLEAR,NVGT,0); // Clear User Selections
	rotate(-90,X);
	mergepoints(0);  // Merge Overlapping Points
	triple();  // Triple all polygons

	testbadpolys = polycount(); // Delete 1 & 2 point polygons
	if((testbadpolys[2] + testbadpolys[3]) > 0)
	{
		selmode(USER);
		selpolygon(SET,NVLT,3);
		cut();                  
	}

	if(invertIt == 1){
		scale(<-1,1,1>);
		flip();
	}

	positively(); // Move so all vertices are represented by positive numbers, as per STL Format Definition
	



// Export STL


	stlFile.write("solid ",stlname);  //_Write Header
	thisFile = Mesh(0);
	thisFileName = thisFile.filename;
	if (thisFileName==nil) {
		thisFileName = "Unknown";
	}

        stlFile.writeln(" created by Export STL and LightWave3D. Original file is ", thisFileName);
		selmode(USER);
		selpolygon(CLEAR);
		selmode(GLOBAL);				

	
		editbegin();
		polynfo = polycount();
	
		moninit(polynfo[1],"Exporting ASCII STL");	
	
		for(loop=1; loop <= polynfo[1]; loop++)	
		{
		
			monstep();
	
			facetNormal = string(polynormal(polygons[loop]));
	
			stlFile.writeln("  facet normal ",facetNormal);
	
			stlFile.writeln("   outer loop");
	
				pointarray = polyinfo(polygons[loop]);
	
				currentpolypoint[1] = string(multiplier * pointinfo(pointarray[2]));
				currentpolypoint[2] = string(multiplier * pointinfo(pointarray[3]));
				currentpolypoint[3] = string(multiplier * pointinfo(pointarray[4]));
	
				for(vertloop = 1; vertloop <=3; ++vertloop)
				{
					stlFile.writeln("    vertex ",currentpolypoint[vertloop]);
				}
	
			stlFile.writeln("   endloop");
	
			stlFile.writeln("  endfacet");
	
		}
	
	
		editend();
		monend();
	
	stlFile.writeln("endsolid ",stlname);  //_Write Closer

// Done exporting the STL. 


	if(invertIt ==1){
		undoIt = 13;
	}
	else{
		undoIt = 11;
	}

	for(undoLoop =1; undoLoop <= undoIt; ++undoLoop){
		undo();
	}
}



//_____USER DEFINED FUNCTIONS

positively
{

	//Moves object into all positive cartesian coordinates


	//_____Rests Object @ X+_________________________

    	bbox = (boundingbox());
    	lovecarray = parse(" ",string(bbox[1]));
    	invrse = - number(lovecarray[1]);
    	move(invrse,0,0);

	//_____Rests Object @ Y+_________________________

    	bbox = (boundingbox());
    	lovecarray = parse(" ",string(bbox[1]));
    	invrse = - number(lovecarray[2]);
    	move(0,invrse,0);

	//_____Rests Object @ Z+_________________________

    	bbox = (boundingbox());
    	lovecarray = parse(" ",string(bbox[1]));
    	invrse = - number(lovecarray[3]);
    	move(0,0,invrse);


return;
}




